@@ -33,10 +33,9 @@ our @ISA = qw(Exporter);
 # This allows declaration	use ASD::Function ':all';
 # If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
 # will save memory.
-our %EXPORT_TAGS = ( 'all' => [ qw( get_title
-				    get_clean_function
-				    remove_man_page_section 
-				    clean_man_page ) ] );
+our %EXPORT_TAGS = ( 'all' => [ qw( 
+	remove_pointless_lines get_title remove_man_page_section clean_man_page
+) ] );
 
 our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 
@@ -46,18 +45,23 @@ our $VERSION = '0.01';
 
 # Preloaded methods go here.
 
-sub get_clean_function {
-    my $filename = shift;
-    
-    my $function = undef;
-    my $extension = undef;
-    my @filename_splitted;
-    
-    @filename_splitted = split( /\./, $filename );
+# Remove comment, empty line and line before of .TH
+sub remove_pointless_lines {
+    my $ref_man_page = shift;
 
-    $function = \&clean_man_page;
+    my $i = 0;
 
-    return $function;
+    while ( $i <= $#$ref_man_page ) {
+	if ( $ref_man_page->[$i] =~ /^\.\\\"|^\.\n|^\n|^\.iX|^\.Vb/ ) {
+	    splice( @$ref_man_page, $i, 1 );
+	} else {
+	    if ( $ref_man_page->[$i] =~ /^\.[Tt][Hh]/ ) {
+		splice( @$ref_man_page, 0, $i );
+		$i = 0;
+	    }
+	    $i++;
+	}
+    }
 }
 
 ##    Get man page title
@@ -67,74 +71,111 @@ sub get_title {
     my $filename = shift;
     my $section = shift;
 
-    my @tmp = undef;
-
-    my $string = " ";
-    my $title = "";
-    my $start_name = 0;
-    my $end_name = 0;
+    my $string = "";
+    my $start_name = -1;
+    my $end_name = -1;
+    my $title = 0;
     my $begin_find = 0;
     my $find = 0;
+    my @tmp = undef;
+    my @tmp2 = undef;
     my $dot = 0; #if title must be cut
 
     #throught man page to find begin NAME section
-    for my $i ( 0.. $#$ref_man_page ) {
+    for my $i ( 0..scalar( @$ref_man_page )-1 ) {
 
-	if ( $ref_man_page->[$i] =~ /^\.th/ ) { #if section name isn't present must use .TH tag
- 	    $title = $ref_man_page->[$i];
- 	} 
-	
-	if ( $ref_man_page->[$i] =~ /^\.sh|^\.ss/ ) { #search begin and end of section name
+	if ( $ref_man_page->[$i] =~ /^\.[S,s]([H,h]|[S,s])/ ) {
+	    
+	    #begin NAME section
+	    if( $ref_man_page->[$i] =~ /NAME/ ) {
 
-  	    #end NAME section
-  	    if ( $begin_find ) {
-  		$end_name = $i-1;
-  		$find = 1;
-  		last;
-  	    }
+		$start_name = $i+1;
+		$begin_find = 1;
+		next;
+	    }
+	    
+	    #end NAME section
+	    if ( $begin_find ) {
 
-  	    #begin NAME section
-  	    if( $ref_man_page->[$i] =~ /name/ ) {
-  		$start_name = $i+1;
-  		$begin_find = 1;
-  	    }
-  	} 
+		$end_name = $i-1;
+		$find = 1;
+		last;
+	    }
+	}
+
+	if (  $ref_man_page->[$i] =~ /^\.[T,t][H,h]/ ) {
+	    $title = $ref_man_page->[$i];
+	}
     }
 
     #if NAME is present can elaborate title
     if ( $find ) {
 
-	for my $i ( $start_name..$end_name ) {
-	    $string = $string."$ref_man_page->[$i] ";
+	#$string = $ref_man_page->[$start_name];
+	for my $i( $start_name .. $end_name ) {
+
+	    $ref_man_page->[$i] =~ s/^\.(\w)+\s|^\.\s|^(\s){1,}//;
+	    $ref_man_page->[$i] =~ s/\\-|\\\(em|\.Nd|--/-/g;
+	    $ref_man_page->[$i] =~ s/\n|\\f.|\\&//g;
+
+	    $string = $string.$ref_man_page->[$i]." ";
 	}
+#	print "\n_{$string}_\n";
+	
+	if( $string ) {
 
-#	print "_{$string}_\n";
-	@tmp = split( /\s-\s/, $string, 2);
+	    #clean title
+	    $string =~ s/\s{2,}/ /g;
+	    $string =~ s/^ //;
 
-	if ( $#tmp ) {
-	    if ( $tmp[1] ) {
-		if ( length( $tmp[1] ) > 200 ) {
-		    $tmp[1] = substr($string, 0, 200) . " ... ";
-		}
-		$string = $filename." ($section) - ".$tmp[1];
-		$string =~ s/^ |$ //g;
+	    #to manage php man page
+ 	    if ($string =~ /^(\d){1,} /) {
+
+ 		$string =~ s/(\d){1,} //;
+
+ 		@tmp = split( / /, $string );
+ 		shift @tmp;
+ 		$string = $filename." -";
+		
+ 		foreach my $i ( @tmp ) {
+ 		    $string .= " ".$i;
+ 		}
+ 	    }
+
+	    @tmp = split( / - /, $string, 2); #divide into 2 parts " - "
+	    @tmp2 = split( /,/ , $tmp[0]);
+
+	    if ( scalar(@tmp2) > 3 ) {
+		$string = $tmp[1];
+		$string = (shift @tmp2).", ..., ".(pop @tmp2)." - ".pop @tmp;
 	    }
-	    else {
-		$string = $filename." ($section) - ".$tmp[0];
-		$string =~ s/^ |$ //g;
+
+	    #add section to title
+	    @tmp = split( / - /, $string, 2 );
+
+	    if ($#tmp > 0) {
+		$string = $filename." ($section) - ".$tmp[1] if ($filename);
+		$string = $tmp[0]." ($section) - ".$tmp[1] unless ($filename);
+	    } else {
+		$string = $tmp[0]." ($section)" unless ($filename);
+		$string = $filename." ($section)" if ($filename);
 	    }
-	}
-	else {
-	    $string = $filename." ($section) - ".$tmp[0];
-	    $string =~ s/^ |$ //g;
+
+	    #delete section NAME
+	    if( $string ) {
+		splice( @$ref_man_page, $start_name-1, ($end_name-$start_name+2) );
+	    }
+
+	    @tmp = ();
+	    @tmp2 = ();
+	    
 	}
 
-	@tmp = ();
     }
     elsif ( $title ) { 
 	$title =~ s/^\.[T,t][H,h] //;
 	$title =~ s/\n|\"//g;
-	$string = $filename." ($section) - ".$title;
+	$string = $filename." - ".$title;
     }
 
     return $string;
@@ -153,7 +194,7 @@ sub remove_man_page_section {
 
     while( $i <= $#$ref_man_page ) {
 
-	if( $ref_man_page->[$i] =~ /^\.sh/ ) {
+	if( $ref_man_page->[$i] =~ /^(\.SH)/ ) {
 
 	    if( $field_to_delete ) {
 
@@ -183,14 +224,13 @@ sub remove_man_page_section {
 sub _apply_reg_exp {
     my $ref_string = shift;
     my $reg_exp = shift;
-    my $sobstitute = shift;
 
     my $before;
     my $after;
 
     do {
 	$before = length( $$ref_string );
-	$$ref_string =~ s/$reg_exp/$sobstitute/g;
+	$$ref_string =~ s/$reg_exp/ /g;
 	$after = length( $$ref_string );
     } while ( $before - $after );
     
@@ -199,6 +239,7 @@ sub _apply_reg_exp {
 #    Clean man page:
 #    - remove all tag of man page
 #    - remove all number sequence
+#    - remove rimuovere tutta la punteggiatura
 #    - double space
 #    - convert all word in down case
 sub clean_man_page {
@@ -210,59 +251,40 @@ sub clean_man_page {
 
 	$ref_man_page->[$i] =~ tr/[A-Z]/[a-z]/;
 	
-	if( $ref_man_page->[$i] =~ /^\.\\\"|^\.\n|^\n|^\.ix|^\.vb/ ) {
+	if( $ref_man_page->[$i] =~ /^\.sh\s|^\.th\s/ ) {
+
 	    splice( @$ref_man_page, $i, 1 );
-	}
-	else {
+	} else {
 	    my $before;
 	    my $after;
 
-	    $ref_man_page->[$i] =~ s/\\f\(cw\\\*\(c|\\\*\(c\'\\fr|\\\*\(r|\\\*\(l|\\f\(cw|\\f.|\\e|\\\(bu|\\s-1|\\s0|\\&//g;  #some tag \tag
+	    $ref_man_page->[$i] =~ s/\\f.|\\e|\\\(bu|\\s-1|\\s0//g;  #some tag \tag
 
-	    $ref_man_page->[$i] =~ s/^\.Nd/-/;
-	    _apply_reg_exp( \$ref_man_page->[$i], "\\\\-|\\\\\\(em|--", "-");
+	    $ref_man_page->[$i] =~ s/^\.(\w){1,}\s/ /;    # tag
+	    $ref_man_page->[$i] =~ s/(\s){1,}/ /g;        # space
+	    $ref_man_page->[$i] =~ s/[^\w|:|-]/ /g;       # tutto cio che non  alfanumerico : -
+	    $ref_man_page->[$i] =~ s/\\-/-/g;
 
- 	    $ref_man_page->[$i] =~ s/^\.(\w)+\s|^\.(\w)+\n|^\.\s//g if( $ref_man_page->[$i] !~ /^\.th\s/ && $ref_man_page->[$i] !~ /^\.s[h,s]\s/ ); # tag .TAG
+	    _apply_reg_exp( \$ref_man_page->[$i], "\\s\\d((\\w)+(\\d)*)+\\s" ); #sequenze di numeri e lettere
+	    _apply_reg_exp( \$ref_man_page->[$i], "^\\d((\\w)+(\\d)*)+\\s" );
 
-	    $ref_man_page->[$i] =~ s/->/ /g;
-	    $ref_man_page->[$i] =~ s/\(|\)/ /g;
- 	    $ref_man_page->[$i] =~ s/[^(\w|:|\-|\.)]/ /g;       # tutto ci che non  alfanumerico : - .
-	    $ref_man_page->[$i] =~ s/:+\s|\s:+/ /g;
- 	    $ref_man_page->[$i] =~ s/\.\s/ /g;
-	    $ref_man_page->[$i] =~ s/\.{2,}/ /g;
+	     _apply_reg_exp( \$ref_man_page->[$i], "\\s(\\d)+-(\\w)+\\s" ); #numero-stringa
+	     _apply_reg_exp( \$ref_man_page->[$i], "^(\\d)+-(\\w)+\\s" );
 
- 	    $ref_man_page->[$i] =~ s/\n|\"|http:\/\///g;
+	    _apply_reg_exp( \$ref_man_page->[$i], "\\s(\\d)+(-(\\d)+)+\\s" ); #numeri intervallati da -
+	    _apply_reg_exp( \$ref_man_page->[$i], "^(\\d)+(-(\\d)+)+\\s" );
 
-	    _apply_reg_exp( \$ref_man_page->[$i], "\\s\\d((\\w)+(\\d)*)+\\s", " " ); #sequenze di numeri e lettere
-	    _apply_reg_exp( \$ref_man_page->[$i], "^\\d((\\w)+(\\d)*)+\\s", " " );
-	    
-	    _apply_reg_exp( \$ref_man_page->[$i], "\\s(\\d)+-(\\w)+\\s", " " ); #numero-stringa
-	    _apply_reg_exp( \$ref_man_page->[$i], "^(\\d)+-(\\w)+\\s", " " );
-		
-	    _apply_reg_exp( \$ref_man_page->[$i], "\\s(\\d)+(-(\\d)+)+\\s", " " ); #numeri intervallati da -
-	    _apply_reg_exp( \$ref_man_page->[$i], "^(\\d)+(-(\\d)+)+\\s", " " );
-	    
-	    _apply_reg_exp( \$ref_man_page->[$i], "\\s(\\d)+(:(\\d)+)+\\s", " " ); #numeri intervallati da : che terminano con un numero
-	    _apply_reg_exp( \$ref_man_page->[$i], "^(\\d)+(:(\\d)+)+\\s", " " );
-	    
-	    _apply_reg_exp( \$ref_man_page->[$i], "\\s((\\d)+:)+\\s", " " );  #numeri intervallati da : che terminano con :
-	    _apply_reg_exp( \$ref_man_page->[$i], "^((\\d)+:)+\\s", " " ); 
-	    
-	    $ref_man_page->[$i] =~ s/^\s+|\s+$//g;
+	    _apply_reg_exp( \$ref_man_page->[$i], "\\s(\\d)+(:(\\d)+)+\\s" ); #numeri intervallati da : che terminano con un numero
+	    _apply_reg_exp( \$ref_man_page->[$i], "^(\\d)+(:(\\d)+)+\\s" );
+
+	    _apply_reg_exp( \$ref_man_page->[$i], "\\s((\\d)+:)+\\s" );  #numeri intervallati da : che terminano con :
+	    _apply_reg_exp( \$ref_man_page->[$i], "^((\\d)+:)+\\s" ); 
+
+	    $ref_man_page->[$i] =~ s/\n|http:\/\///;
+
+	    $ref_man_page->[$i] =~ s/^ {1,}| {1,}$//g;
 	    $ref_man_page->[$i] =~ s/ {2,}/ /g;
 
-
-	    if ( $ref_man_page->[$i] =~ /^\.th\s/ ) {
-		splice( @$ref_man_page, 0, $i );
-		$i = 1;
-		next;
-	    }
-	    
-	    if ( $ref_man_page->[$i] =~ /^\.sh\s/ ) {
-		$i++;
-	        next;
-	    }
-
 	    if ( $ref_man_page->[$i] ) {
 		$i++;
 	    } else {
@@ -270,6 +292,7 @@ sub clean_man_page {
 	    }
 	}
     }
+
 }
 
 
